/*
 *  Engine/Settings.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Collections;
using System.Xml;
using System.IO;


namespace esofokles.Engine {
	public struct Option {
		public string key;
		public string val;
	}
	

	/*
	 * Settings class
	 */
	
	public class Settings {
		private static ArrayList Data;

		static public bool Open ()
		{
			Data = new ArrayList ();
			
			StreamReader reader;
			
			try {
				reader = new StreamReader (FilesPaths.SettingsFile (), System.Text.Encoding.UTF8);
			} catch  {
				LoadDefaultSettings ();
				return false;
			}
			
			XmlTextReader read = new XmlTextReader (reader);
			Option        opt  = new Option ();
			
			try {
				while (read.Read ())
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement))
						if ((read.Name == "option") && (read.NodeType == XmlNodeType.Element)) {
							opt     = new Option ();
							opt.key = read.GetAttribute ("key");
							opt.val = read.ReadString ();

							Data.Add (opt);
						}
			} catch {
				LoadDefaultSettings ();
				reader.Close ();
				return false;
			}
			

			reader.Close ();
			return true;
		}


		static public void Close ()
		{
			// save settings
			XmlTextWriter f = new XmlTextWriter
				(FilesPaths.SettingsFile (), System.Text.Encoding.UTF8);

			f.Formatting = Formatting.Indented;
			f.WriteStartDocument ();
			f.WriteStartElement (null, "esofoklessettings", "");
				foreach (Option opt in Data) {
					f.WriteStartElement ("option");
					f.WriteAttributeString ("key", opt.key);
						f.WriteString (opt.val);
					f.WriteEndElement ();
				}
			f.WriteEndElement ();

			f.Close ();
		}
		
		
		static public string Get (string key)
		{
			foreach (Option opt in Data) {
				if (opt.key == key)
					return opt.val;
			}

			return "";
		}
		
		
		static public bool GetBool (string key)
		{
			if (Get (key) == "")
				return true;			
			
			return System.Convert.ToBoolean (Get (key));
		}
		
		
		static public bool GetBoolDF (string key)
		{
			if (Get (key) == "")
				return false;			
			
			return System.Convert.ToBoolean (Get (key));
		}
		
		
		static public void Set (string key, string val)
		{
			int i = 0;
			
			foreach (Option opt in Data) {
				if (opt.key == key) {
					Option newopt = new Option ();
					newopt.key = key;
					newopt.val = val;
					Data [i] = newopt;
					return;
				}

				i++;
			}


			Option nopt = new Option ();
			nopt.key    = key;
			nopt.val    = val;
			
			Data.Add (nopt);

			return;
		}
		
		
		static public void SetBool (string key, bool val)
		{
			Set (key, val.ToString ());
		}
		
		
		static public void LoadDefaultSettings ()
		{
		}
	}
}
