/*
 *  GUI/ClassDialog.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Data;
using System.Data.SqlTypes;
using Gtk;
using Gdk;
using esofokles.Engine;
using Mono.Unix;
using esofokles.Widgets;


namespace esofokles.GUI {

	/// <summary>Class dialog</summary>
	public class ClassDialog {
		public Gtk.Dialog dialog;

		[Glade.Widget] private Gtk.Viewport   Bg;
		[Glade.Widget] public  Gtk.Entry      Token, Description;
		[Glade.Widget] public  Gtk.SpinButton ClassNumber, SchoolYear, SchoolYear2;
		[Glade.Widget] private Gtk.Button     CancelBtn, RemoveStudentBtn,
											  MoveStudentBtn, OpenStudentBtn,
											  ReportsBtn, SubjectsListBtn, AddMarksBtn;
		[Glade.Widget] private Gtk.Frame      StudentsFrame, ClassFrame;
		[Glade.Widget] private Gtk.TreeView   StudentsList;
		[Glade.Widget] private Gtk.Label      ClassName, ClassDescription;

		public  int      ClassID;
		private Button24 OkBtn;

		
		/// <summary>Constructor. Creates class dialog.</summary>
		public ClassDialog (Gtk.Window parent, int ClassID)
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "ClassDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("ClassDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			dialog.DestroyWithParent = true;
			
			this.ClassID = ClassID;
			
			
			// OkBtn
			OkBtn = new Button24 (Catalog.GetString ("OK"));
			OkBtn.Clicked += new EventHandler (on_OkBtn_clicked);
			dialog.ActionArea.Add (OkBtn);
			OkBtn.ShowAll ();
			
			
			// Icons and colors
			Icons.AssignWindow (dialog);
			
			Icons.AssignImage (Icons.ADD_STUDENT_BUTTON, "AddStudentBtnImg", gui);
			Icons.AssignImage (Icons.OPEN_STUDENT_BUTTON, "OpenStudentBtnImg", gui);
			Icons.AssignImage (Icons.REMOVE_STUDENT_BUTTON, "RemoveStudentBtnImg", gui);
			Icons.AssignImage (Icons.MOVE_STUDENT_BUTTON, "MoveStudentBtnImg", gui);
			Icons.AssignImage (Icons.OK_BUTTON, OkBtn.Img);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			Icons.AssignImage (Icons.SUBJECTS_LIST_BUTTON, "SubjectsListBtnImg", gui);
			Icons.AssignImage (Icons.REPORTS_BUTTON, "ReportsBtnImg", gui);
			
			Icons.AssignImage (Icons.CLASS_HEAD, "Icon", gui);
			
			Colors.Modify (Colors.HEAD, Bg);
			
			
			StudentsListCreate ();
			Tools.SetSensitive (dialog, false);
			

			if (ClassID == -1) {
				CancelBtn.Show ();
				SetSensitive ();
				SchoolYear.Value  = Tools.CurrentSchoolYear ();
				SchoolYear2.Value = SchoolYear.Value + 1;
			} else {
				ClassButtonsSetSensitive (true);
				CancelBtn.Hide ();
				
				
				IDataReader reader = DB.Query (String.Format ("SELECT * FROM " +
									"classes WHERE id LIKE '{0}';", ClassID));
				if (reader != null) {
					reader.Read ();
					ClassNumber.Value = (int) reader ["number"];
					Token.Text = (string) reader ["token"];
					Description.Text  = (string) reader ["description"];
					SchoolYear.Value  = (int) reader ["year"];
					SchoolYear2.Value = SchoolYear.Value + 1;
				}

				reader.Close ();
				
				ReloadStudentsList ();
				SetSensitive ();
			}
			
			
			// Autocompletion
			Autocompletion.Assign (Description, Field.ClassesDescription);
			
			Tools.SetSensitive (dialog, true);
		}
		
		
		protected void on_OkBtn_clicked (object o, EventArgs args)
		{
			Autocompletion.AddText (Description, Field.ClassesDescription);
			
			IDataReader reader;
			reader = DB.Query (String.Format ("SELECT * FROM classes WHERE " +
						"number = '{0}' AND token = '{1}' AND year = '{2}' AND " +
						"id != '{3}';", ClassNumber.Value, DB.Escape (Token.Text),
						SchoolYear.Value, ClassID));
			
			if (reader != null) {
				if (reader.Read ()) {
					int id = (int) reader ["id"];
					if (id != -1) {
						Tools.SetSensitive (dialog, false);
						
						MessageDialog msg = new MessageDialog (dialog,
								DialogFlags.DestroyWithParent, MessageType.Warning,
								ButtonsType.Ok, Catalog.GetString (
								"Class with the same name already " +
								"exists !\nChange class name."));
						msg.Run ();
						msg.Destroy ();
						reader.Close ();
						
						Tools.SetSensitive (dialog, true);
						return;
					}
				}
				
				reader.Close ();
			}
			
			dialog.Respond (ResponseType.Ok);
		}


		protected void on_ClassNumber_value_changed (object o, EventArgs args)
		{
			ClassName.Markup = String.Format ("<span weight=\"bold\" size=\"x-" +
						"large\">{0} {1}</span>", ClassNumber.Value, Token.Text);
		}


		protected void on_Token_changed (object o, EventArgs args)
		{
			SetSensitive ();
			ClassName.Markup = String.Format ("<span weight=\"bold\" size=\"x-" +
						"large\">{0} {1}</span>", ClassNumber.Value, Token.Text);
		}


		protected void on_Description_changed (object o, EventArgs args)
		{
			SetSensitive ();
			ClassDescription.Text = Catalog.GetString ("Educator") + ": " + Description.Text;
		}


		protected void on_StudentsList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (StudentsList.Selection.GetSelected (out mod, out itr))
				StudentsButtonsSetSensitive (true);
			else
				StudentsButtonsSetSensitive (false);
		}


		protected void on_AddStudentBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			
			if (ClassID == -1) {
				// Does selected class exists ?
				IDataReader reader;
				reader = DB.Query (String.Format ("SELECT * FROM classes WHERE " +
							"number = '{0}' AND token = '{1}' AND year = '{2}' AND " +
							"id != '{3}';", ClassNumber.Value, DB.Escape (Token.Text),
							SchoolYear.Value, ClassID));
				
				if (reader != null) {
					if (reader.Read ()) {
						int id = (int) reader ["id"];
						if (id != -1) {
							Tools.SetSensitive (dialog, false);
							
							MessageDialog msg = new MessageDialog (dialog,
									DialogFlags.DestroyWithParent, MessageType.Warning,
									ButtonsType.Ok, "Class with the same name already " +
													"exists !\nChange class name.");
							msg.Run ();
							msg.Destroy ();
							reader.Close ();
							
							Tools.SetSensitive (dialog, true);
							return;
						}
					}
					
					reader.Close ();
				}
				
				
				DB.FastQuery (String.Format ("INSERT INTO classes (year, description, number, token) VALUES (" +
						"'{0}', '{1}', '{2}', '{3}');", SchoolYear.Value.ToString (),
						DB.Escape (Description.Text), ClassNumber.Value.ToString (),
						DB.Escape (Token.Text)));
				
				IDataReader read = DB.Query (String.Format (
					"SELECT id from classes where (year = '{0}') AND (number = " +
					"'{1}') AND (token = '{2}');", SchoolYear.Value.ToString (),
					ClassNumber.Value.ToString (), DB.Escape (Token.Text)));
				
				if (read != null) {
					read.Read ();
					ClassID  = (int) read ["id"];
					
					CancelBtn.Hide ();
					read.Close ();
				} else
					return;
			}

			
			StudentDialog dlg = new StudentDialog (this.dialog, -1, ClassID);
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				if (dlg.StudentID == -1) {
					DB.FastQuery (String.Format ("INSERT INTO students (class, name, surname, pesel, sid, birthdate, birthplace, address, parentsaddress, telephone, father, mother, info) VALUES (" +
						"'{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}', '{7}', " +
						"'{8}', '{9}', '{10}', '{11}', '{12}');",
						ClassID, DB.Escape (dlg.Name.Text), DB.Escape (dlg.Surname.Text),
						DB.Escape (dlg.Pesel.Text), DB.Escape (dlg.SID.Text),
						DB.Escape (dlg.BirthDate.Text), DB.Escape (dlg.BirthPlace.Text),
						DB.Escape (dlg.Address.Text), DB.Escape (dlg.ParentsAddress.Text),
						DB.Escape (dlg.Telephone.Text), DB.Escape (dlg.Father.Text),
						DB.Escape (dlg.Mother.Text), DB.Escape (dlg.Info.Buffer.Text)));
				} else {
					DB.FastQuery (String.Format ("UPDATE students SET name='{0}', s" +
						"urname='{1}', pesel='{2}', sid='{3}', birthdate='{4}', b" +
						"irthplace='{5}',  address='{6}', parentsaddress='{7}', " +
						"telephone='{8}', father='{9}', mother='{10}', info='{11}'" +
						"WHERE id = '{12}';", DB.Escape (dlg.Name.Text), DB.Escape (dlg.Surname.Text),
						DB.Escape (dlg.Pesel.Text), DB.Escape (dlg.SID.Text),
						DB.Escape (dlg.BirthDate.Text), DB.Escape (dlg.BirthPlace.Text),
						DB.Escape (dlg.Address.Text), DB.Escape (dlg.ParentsAddress.Text),
						DB.Escape (dlg.Telephone.Text), DB.Escape (dlg.Father.Text),
						DB.Escape (dlg.Mother.Text), DB.Escape (dlg.Info.Buffer.Text),
						dlg.StudentID));
				}
				
				ReloadStudentsList ();
			}
			
			dlg.dialog.Destroy ();
			
			
			StudentsButtonsSetSensitive (false);
			SetSensitive ();
			Tools.SetSensitive (dialog, true);
		}


		protected void on_RemoveStudentBtn_clicked (object o, EventArgs args)
        {
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (StudentsList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				Gtk.ListStore store = (Gtk.ListStore) StudentsList.Model;
				
				MessageDialog msg = new MessageDialog (this.dialog, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, String.Format (Catalog.GetString ("Are you really sure to remove <b>{0} {1}</b> student ?"),
									(string) store.GetValue (itr, 0), (string) store.GetValue (itr, 1)));
				
				if (ResponseType.Yes == (ResponseType) msg.Run ()) {
					StudentsButtonsSetSensitive (false);
					
					string query = "";
					query += String.Format ("DELETE FROM students WHERE id = '{0}';", (int) store.GetValue (itr, 2));
					query += String.Format ("DELETE FROM attendance WHERE student = '{0}';", (int) store.GetValue (itr, 2));
					query += String.Format ("DELETE FROM marks WHERE student = '{0}';", (int) store.GetValue (itr, 2));
					query += String.Format ("DELETE FROM remarks WHERE student = '{0}';", (int) store.GetValue (itr, 2));
					query += String.Format ("DELETE FROM semestermarks WHERE student = '{0}';", (int) store.GetValue (itr, 2));
					DB.FastQuery (query);

					store.Remove (ref itr);
				}

				msg.Destroy ();
				Tools.SetSensitive (dialog, true);
			}
		}

	
		protected void on_MoveStudentBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (StudentsList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				
				MoveStudentDialog dlg =  new MoveStudentDialog (this.dialog, (string) StudentsList.Model.GetValue (itr, 0) + " " +
						(string) StudentsList.Model.GetValue (itr, 1), ClassID);

				
				if (dlg.dialog.Run () == (int) ResponseType.Ok) {
					int StudentID = (int) StudentsList.Model.GetValue (itr, 2);

					Gtk.TreeIter iter;
					if (dlg.MoveClass.Active) {
						if (dlg.ClassesList.GetActiveIter (out iter))  {
							int NewClass = SearchClass ((string) dlg.ClassesList.Model.GetValue (iter, 0));
							
							string query = "";
							query += String.Format ("UPDATE students SET class " +
									"= '{0}' WHERE id = '{1}';", NewClass, StudentID);
							query += String.Format ("DELETE FROM attendance WHERE " +
												"student = '{0}';", StudentID);
	
							if (dlg.ImportMarks.Active == true) {
								query += String.Format ("update marks,subjects " +
									"set marks.subject = subjects.id  where " +
									"UPPER(subjects.subject) = ANY(select UPPER(subject) from " +
									"subjects where id=marks.subject) and marks." +
									"student = '{0}' and subjects.class='{1}';",
									StudentID, NewClass);
								
								query += String.Format ("update semestermarks,subjects " +
									"set semestermarks.subject = subjects.id  where " +
									"UPPER(subjects.subject) = ANY(select UPPER(subject) from " +
									"subjects where id=semestermarks.subject) and semestermarks." +
									"student = '{0}' AND subjects.class='{1}';",
									StudentID, NewClass);
								
								query += String.Format ("DELETE FROM marks WHERE " +
									"subject = ANY(select id from subjects where " +
									"class='{0}') AND student = '{1}';", dlg.ClassID, StudentID);
								
								query += String.Format ("DELETE FROM semestermarks WHERE " +
									"subject = ANY(select id from subjects where " +
									"class='{0}') AND student = '{1}';", dlg.ClassID, StudentID);
							} else {
								query += String.Format ("DELETE FROM marks WHERE " +
												"student = '{0}';", StudentID);
								query += String.Format ("DELETE FROM semestermarks WHERE " +
												"student = '{0}';", StudentID);
							}
							
							if (dlg.ImportRemarks.Active == false)
								query += String.Format ("DELETE FROM remarks WHERE " +
												"student = '{0}';", StudentID);
							
							DB.FastQuery (query);
						}
					} else
						if (dlg.OtherClassesList.GetActiveIter (out iter)) {
							string query = String.Format ("INSERT INTO students (class, name, " +
								"surname, pesel, sid, birthdate, birthplace, address, " +
								"parentsaddress, telephone, father, mother, info) " +
								"(select '{0}', name, surname, pesel, sid, birthdate, " +
								"birthplace, address, parentsaddress, telephone, father, " +
								"mother, info from students where id = '{1}');",
								SearchClass ((string) dlg.OtherClassesList.Model.GetValue (iter, 0)),
								StudentID);
								
							query += String.Format ("DELETE FROM students WHERE " +
												"id = '{0}';", StudentID);
							
							DB.FastQuery (query);
						}

					ReloadStudentsList ();
				}
				
				dlg.dialog.Destroy ();
				Tools.SetSensitive (dialog, true);
			}
		}


		protected void on_StudentsList_row_activated (object o, RowActivatedArgs args)
		{
			on_OpenStudentBtn_clicked (null, null);
		}


		protected void on_OpenStudentBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (StudentsList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				StudentDialog dlg = new StudentDialog (this.dialog,
							(int) (mod as ListStore).GetValue (itr, 2), ClassID);
				
				if (dlg.dialog.Run () == (int) ResponseType.Ok) {
					DB.FastQuery (String.Format ("UPDATE students SET name='{0}', s" +
						"urname='{1}', pesel='{2}', sid='{3}', birthdate='{4}', b" +
						"irthplace='{5}',  address='{6}', parentsaddress='{7}', " +
						"telephone='{8}', father='{9}', mother='{10}', info='{11}'" +
						"WHERE id = '{12}';", DB.Escape (dlg.Name.Text), DB.Escape (dlg.Surname.Text),
						DB.Escape (dlg.Pesel.Text), DB.Escape (dlg.SID.Text), DB.Escape (dlg.BirthDate.Text),
						DB.Escape (dlg.BirthPlace.Text), DB.Escape (dlg.Address.Text), DB.Escape (dlg.ParentsAddress.Text),
						DB.Escape (dlg.Telephone.Text), DB.Escape (dlg.Father.Text), DB.Escape (dlg.Mother.Text),
						DB.Escape (dlg.Info.Buffer.Text), dlg.StudentID));
					
					ReloadStudentsList ();
				}
				
				dlg.dialog.Destroy ();
				Tools.SetSensitive (dialog, true);
				
				StudentsButtonsSetSensitive (false);
			}
		}


		protected void on_ReportsBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			
			ReportsDialog dlg = new ReportsDialog (this.dialog, ClassID);
			dlg.dialog.Run ();
			dlg.dialog.Destroy ();
			
			Tools.SetSensitive (dialog, true);
		}


		protected void on_SubjectsListBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			SubjectsListDialog dlg = new SubjectsListDialog (this.dialog, ClassID);
			dlg.dialog.Run ();
			dlg.dialog.Destroy ();
			Tools.SetSensitive (dialog, true);
		}
		
		
		protected void on_AddMarksBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			
			AddingMarksDialog dlg = new AddingMarksDialog (dialog, ClassID);
			dlg.dialog.Run ();
			dlg.dialog.Destroy ();
			Tools.SetSensitive (dialog, true);
		}
		
		
		protected void on_SchoolYear_value_changed (object o, EventArgs args)
		{
			SchoolYear2.Value = SchoolYear.Value + 1;
		}
		
		
		protected void on_SchoolYear2_value_changed (object o, EventArgs args)
		{
			SchoolYear.Value = SchoolYear2.Value - 1;
		}
		
		
		/// <summary>Sets students buttons sensitive.</summary>
		private void StudentsButtonsSetSensitive (bool sensitive)
		{
			RemoveStudentBtn.Sensitive = sensitive;
			MoveStudentBtn.Sensitive   = sensitive;
			OpenStudentBtn.Sensitive   = sensitive;
		}


		/// <summary>Sets class buttons sensitive.</summary>
		private void ClassButtonsSetSensitive (bool sensitive)
		{
			ReportsBtn.Sensitive = sensitive;
			AddMarksBtn.Sensitive = sensitive;
			SubjectsListBtn.Sensitive = sensitive;
		}
		
		
		/// <summary>Sets sensitive.</summary>
		private void SetSensitive ()
		{
			if (Token.Text != "") {
				if (ClassID == -1)
					OkBtn.Sensitive = true;
				else
					OkBtn.Sensitive = true;

				StudentsFrame.Sensitive = true;
				ClassFrame.Sensitive    = (ClassID != -1);
			} else {
				if (ClassID == -1) {
					OkBtn.Sensitive = false;
				} else
					OkBtn.Sensitive = false;

				StudentsFrame.Sensitive = false;
				ClassFrame.Sensitive    = false;
			}
		}


		/*
		 * STUDENTS LIST
		 */

		/// <summary>Creates students list.</summary>
		private void StudentsListCreate ()
		{
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (string), typeof (int));
			store.SetSortColumnId (1, Gtk.SortType.Ascending);
			StudentsList.Model = store;

			StudentsList.AppendColumn (Catalog.GetString ("First name"), new CellRendererText (), "text", 0);
			StudentsList.AppendColumn (Catalog.GetString ("Last name"), new CellRendererText (), "text", 1);

			StudentsList.Columns [0].Clickable = true;
			StudentsList.Columns [0].SortIndicator = true;
			StudentsList.Columns [0].SortColumnId = 0;

			StudentsList.Columns [1].Clickable = true;
			StudentsList.Columns [1].SortIndicator = true;
			StudentsList.Columns [1].SortColumnId = 1;
		}
		
		
		/// <summary>Reloads students list.</summary>
		private void ReloadStudentsList ()
		{
			Tools.SetSensitive (dialog, false);
			
			// Updating students list
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM students WHERE class = {0};",
						ClassID));
			
			if (reader != null) {
				Gtk.ListStore StudentsListStore = (Gtk.ListStore) StudentsList.Model;
				StudentsListStore.Clear ();
				while (reader.Read ())
					StudentsListStore.AppendValues (new object [] {(string) reader ["name"], (string) reader ["surname"],
											(int) reader ["id"]});

				reader.Close ();
			}

			Tools.SetSensitive (dialog, true);
		}
		
		
		/// <summary>Searches for class by its full name (number token).</summary>
		private int SearchClass (string ClassFullName)
		{
			string number = "",  token = "";
			for (int i = 0; i < ClassFullName.Length; i++)
				if (ClassFullName [i] == ' ') {
					number = ClassFullName.Substring (0, i);
					token  = ClassFullName.Substring (i + 1, ClassFullName.Length - i - 1);
					break;
				}
		
			
			IDataReader reader;
			reader = DB.Query (String.Format ("SELECT * FROM classes WHERE number = '{0}' AND token = '{1}' AND year = '{2}';",
						number, DB.Escape (token), SchoolYear.Value));
			
			if (reader != null) {
				if (reader.Read ()) {
					int id = (int) reader ["id"];
					reader.Close ();
					return id;
				}
				
				reader.Close ();
			}
			
			return -1;
		}
	}
}
