/*
 *  GUI/MainWindow.cs
 *  Copyright (C) 2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


// beta version
//#define beta
#define NoSupport



using System;
using System.Reflection;
using System.Data;
using System.Collections;
using Gdk;
using Gtk;
using Glade;
using esofokles.Engine;
using esofokles.Widgets;
using Mono.Unix;



namespace esofokles.GUI {
	
	/// <summary>Main window</summary>
	public class MainWindow {
		private Glade.XML gui;
		private Gtk.Window window;

		[Glade.Widget] Gtk.Notebook    MainNotebook;
		[Glade.Widget] Gtk.Label       UserName;
		[Glade.Widget] Gtk.Entry       DBServer, DBName, DBPort, DBUser, DBPassword;
		[Glade.Widget] Gtk.Button      LoginBtn, RemoveClass, OpenClass;
		[Glade.Widget] Gtk.TreeView    ClassesList;
		[Glade.Widget] Gtk.VBox        UserBox;
		[Glade.Widget] Gtk.CheckButton ShowOnlyCurrentSchoolYear;
		[Glade.Widget] Gtk.ImageMenuItem support;
		
		
		/// <summary>Constructor. Creates MainWindow.</summary>
		public MainWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "MainWindow", null);
			gui.Autoconnect (this);
			
			window = (Gtk.Window) gui.GetWidget ("MainWindow");
			MainNotebook.ShowTabs = false;
			
			
			// Icons and colors
			Icons.AssignWindow (window);
			
			Icons.AssignImage (Icons.ADD_CLASS_BUTTON, "AddClassBtnImg", gui);
			Icons.AssignImage (Icons.REMOVE_CLASS_BUTTON, "RemoveClassBtnImg", gui);
			Icons.AssignImage (Icons.OPEN_CLASS_BUTTON, "OpenClassBtnImg", gui);
			Icons.AssignImage (Icons.LOGIN_BUTTON, "LoginBtnImg", gui);
			Icons.AssignImage (Icons.LOGOUT_BUTTON, "LogoutBtnImg", gui);
			Icons.AssignImage (Icons.IMPORT_CLASSES_BUTTON, "ImportClassesBtnImg", gui);
			
			Icons.AssignMenuItem (Icons.LINK_MENU, "esofokles_online", gui);
			Icons.AssignMenuItem (Icons.LINK_MENU, "abcom_online", gui);
			Icons.AssignMenuItem (Icons.LINK_MENU, "support", gui);
			
			Colors.Modify (Colors.HEAD, "UserViewport", gui);
			
			
		#if NoSupport
			support.Hide();
		#endif
			
			
			// filling server data fields from settings
			DBServer.Text = Engine.Settings.Get ("DBServer");
			DBName.Text   = Engine.Settings.Get ("DBName");
			DBPort.Text   = Engine.Settings.Get ("DBPort");
			DBUser.Text   = Engine.Settings.Get ("DBUser");

			if ((DBServer.Text != "") && (DBName.Text != "") && (DBUser.Text != ""))
				window.Focus = DBPassword;
			
			
			// class list
			ClassesListCreate ();
			new MessageBox (MessageBox.MsgType.info, false, Catalog.GetString
					("If you want to import classes from previous school year click on <u>Import</u> button.")).AddToBox ("vbox11", gui);
			
			
		#if beta
			ShowBetaVersionDialog ();
		#endif
		}


		/// <summary>On delete MainWindow event</summary>
		protected void on_MainWindow_delete_event (object o, DeleteEventArgs args)
		{
			on_exit_activate (null, null);
		}


	#if beta
		/// <summary>Shows beta versin dialgo</summary>
		private void ShowBetaVersionDialog ()
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "BetaVersionDialog", null);
			gui.Autoconnect (this);
			
			Gtk.Dialog dlg = (Gtk.Dialog) gui.GetWidget ("BetaVersionDialog");
			dlg.Icon = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "esofokles.png");
			
			(gui.GetWidget ("BetaVersionDialogVBox") as Gtk.VBox).BorderWidth = 0;
			Icons.AssignImage (Icons.ABOUT_HEAD, "Banner", gui);
			(gui.GetWidgetesofokles_online, abcom_online ("Banner") as Gtk.Image).Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "about.png");
			(gui.GetWidget ("CloseBtnImg") as Gtk.Image).Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "cancel.png");
			(gui.GetWidget ("Bg") as Gtk.Viewport).ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			
			
			dlg.Run ();
			dlg.Destroy ();
		}
	#endif
	
		
		/// <summary>Exit menu item clicked.</summary>
		protected void on_exit_activate (object o, EventArgs args)
		{
			Engine.Settings.SetBool ("ShowOnlyCurrentSchoolYear", ShowOnlyCurrentSchoolYear.Active);
			Engine.Settings.Close (); // saving settings
			Engine.DB.Disconnect ();  // closing DB connection
			Application.Quit ();
		}
		
		
		protected void on_esofokles_online_activate (object o, EventArgs args)
		{
			Tools.RunWebBrowser ("http://www.esofokles.ab-com.pl/");
		}
		
		
		protected void on_abcom_online_activate (object o, EventArgs args)
		{
			Tools.RunWebBrowser ("http://www.ab-com.pl/");
		}
		
		
		protected void on_support_activate (object o, EventArgs args)
		{
			Tools.RunWebBrowser ("http://www.support.ab-com.pl/esofokles/");
		}

		
		/// <summary>About menu item clicked.</summary>
		protected void on_about_activate (object o, EventArgs args)
		{
			AboutDialog dlg = new AboutDialog (this.window);
			window.Sensitive = false;
			dlg.dialog.Run ();
			dlg.dialog.Destroy ();
			window.Sensitive = true;
		}


		/// <summary>On switch MainNotebook page. Reloads classes list if new page = 1</summary>
		protected void on_MainNotebook_switch_page (object o, SwitchPageArgs args)
		{
			if (MainNotebook.Page == 1)
				ClassesListReload (true);
		}


		/// <summary>Changes login button sensitive./summary>
		private void DBRemoteSensitive ()
		{
			if ((DBName.Text != "") && (DBUser.Text != ""))
				LoginBtn.Sensitive = true;
			else
				LoginBtn.Sensitive = false;
		}


		/// <summary>Sets focus in DB form.</summary>
		private bool DBRemoteSetFocus ()
		{
			if (DBName.Text == "") {
				window.Focus = DBName;
				return true;
			}

			if (DBUser.Text == "") {
				window.Focus = DBUser;
				return true;
			}

			return false;
		}


		/// <summary>On pressed enter key in server entry.</summary>
		protected void on_DBServer_activate (object o, EventArgs args)
		{
			if (DBRemoteSetFocus () == false)
				on_LoginBtn_clicked (null, null);
		}


		/// <summary>On changed database name entry.</summary>
		protected void on_DBName_changed (object o, EventArgs args)
		{
			DBRemoteSensitive ();
		}


		/// <summary>On pressed enter key in name entry.</summary>
		protected void on_DBName_activate (object o, EventArgs args)
		{
			if (DBRemoteSetFocus () == false)
				on_LoginBtn_clicked (null, null);
		}

		
		/// <summary>On pressed enter key in port entry.</summary>
		protected void on_DBPort_activate (object o, EventArgs args)
		{
			if (DBRemoteSetFocus () == false)
				on_LoginBtn_clicked (null, null);
		}


		/// <summary>On changed database user name entry.</summary>
		protected void on_DBUser_changed (object o, EventArgs args)
		{
			DBRemoteSensitive ();
		}


		/// <summary>On pressed enter key in user name entry.</summary>
		protected void on_DBUser_activate (object o, EventArgs args)
		{
			if (DBRemoteSetFocus () == false)
				on_LoginBtn_clicked (null, null);
		}


		/// <summary>On pressed enter key in password entry.</summary>
		protected void on_DBPassword_activate (object o, EventArgs args)
		{
			if (DBRemoteSetFocus () == false)
				on_LoginBtn_clicked (null, null);
		}


		/// <summary>On pressed login button.</summary>
		protected void on_LoginBtn_clicked (object o, EventArgs args)
		{
			// saving db settings
			Engine.Settings.Set ("DBServer", DBServer.Text);
			Engine.Settings.Set ("DBName", DBName.Text);
			Engine.Settings.Set ("DBPort", DBPort.Text);
			Engine.Settings.Set ("DBUser", DBUser.Text);


			// connecting
			Tools.SetSensitive (window, false);
			
			Engine.DB.Create (DBType.MySQL, DBServer.Text, DBUser.Text, DBPassword.Text, DBPort.Text, DBName.Text);
			
			
			if (Engine.DB.Connect () == true) {
				UserName.Text = DBUser.Text;
				DBPassword.Text = "";
				MainNotebook.Page = 1;
				UserBox.Show ();
				
				// Init autocompletion
				Autocompletion.Init ();
				
				ShowOnlyCurrentSchoolYear.Active = Engine.Settings.GetBool ("ShowOnlyCurrentSchoolYear");
				Tools.SetSensitive (window, true);
			} else {
				MessageDialog md = new MessageDialog (window, DialogFlags.DestroyWithParent, MessageType.Error, ButtonsType.Close,
						Catalog.GetString ("Could not connect to the database.\n"
						+ "Check your user name and password."));
				md.Run ();
				md.Destroy();

				Tools.SetSensitive (window, true);
				window.Focus = DBPassword;
			}
		}


		/// <summary>On Log out button clicked.</summary>
		protected void on_LogoutBtn_clicked (object o, EventArgs args)
		{
			Engine.Settings.SetBool ("ShowOnlyCurrentSchoolYear", ShowOnlyCurrentSchoolYear.Active);
			Engine.DB.Disconnect ();
			MainNotebook.Page = 0;
			UserBox.Hide ();
			window.Focus = DBPassword;
		}



		/*
		 * Classes list
		 */
		
		/// <summary>Creates classes list.</summary>
		private void ClassesListCreate ()
		{
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (int), typeof (string), typeof (string), typeof (int));
			store.SetSortColumnId (0, Gtk.SortType.Descending);
			ClassesList.Model = store;

			ClassesList.AppendColumn (Catalog.GetString ("School year"), new CellRendererText  (), "markup", 0);
			ClassesList.AppendColumn (Catalog.GetString ("Class"), new CellRendererText (), "text", 1);
			ClassesList.AppendColumn (Catalog.GetString ("Token"), new CellRendererText (), "text", 2);
			ClassesList.AppendColumn (Catalog.GetString ("Educator"), new CellRendererText (), "text", 3);

			ClassesList.Columns [0].Clickable = true;
			ClassesList.Columns [0].SortIndicator = true;
			ClassesList.Columns [0].SortColumnId = 0;

			ClassesList.Columns [1].Clickable = true;
			ClassesList.Columns [1].SortIndicator = true;
			ClassesList.Columns [1].SortColumnId = 1;

			ClassesList.Columns [2].Clickable = true;
			ClassesList.Columns [2].SortIndicator = true;
			ClassesList.Columns [2].SortColumnId = 2;

			ClassesList.Columns [3].Clickable = true;
			ClassesList.Columns [3].SortIndicator = true;
			ClassesList.Columns [3].SortColumnId = 3;
		}
		
		
		/// <summary>Reloads classes list.</summary>
		private void ClassesListReload (bool CurrentYear)
		{
			Tools.SetSensitive (window, false);
			
			Gtk.ListStore store  = (Gtk.ListStore) ClassesList.Model;
			IDataReader reader;

			if (CurrentYear == true)
				reader = DB.Query (String.Format ("SELECT * FROM classes WHERE year = '{0}';", Tools.CurrentSchoolYear ()));
			else
				reader = DB.Query ("SELECT * FROM classes;");
			
			string schyr = "";
			if (reader != null) {
				store.Clear ();
				
				while (reader.Read ()) {
					if ((int) reader ["year"] == Tools.CurrentSchoolYear ())
						schyr = "<b>" + Tools.SchoolYear ((int) reader ["year"]) + "</b>";
					else
						schyr = Tools.SchoolYear ((int) reader ["year"]);
				
					store.AppendValues (new object [] {schyr, (int) reader ["number"],
							(string) reader ["token"], (string) reader ["description"], (int) reader ["id"]});
				}
				
				reader.Close ();
			}
			
			Tools.SetSensitive (window, true);
		}

		
		/// <summary>On Show only current school year toggled.</summary>
		protected void on_ShowOnlyCurrentSchoolYear_toggled (object o, EventArgs args)
		{
			ClassesListReload (ShowOnlyCurrentSchoolYear.Active);
			ClassesButtonsSetSensitive (false);
			
			this.window.Focus = ShowOnlyCurrentSchoolYear;
		}

	
		/// <summary>Sets classes list sensitive.</summary>
		private void ClassesButtonsSetSensitive (bool sensitive)
		{
			OpenClass.Sensitive   = sensitive;
			RemoveClass.Sensitive = sensitive;
		}
		
		
		/// <summary>On add class button clicked.</summary>
		protected void on_AddClass_clicked (object o, EventArgs args)
		{
			ClassesButtonsSetSensitive (false);
			Tools.SetSensitive (window, false);
			
			ClassDialog dlg = new ClassDialog (this.window, -1);
			
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				if (dlg.ClassID == -1) {
					DB.FastQuery (String.Format ("INSERT INTO classes (year, description, number, token) VALUES ('{0}', '{1}', '{2}', '{3}');",
								dlg.SchoolYear.Value.ToString (), DB.Escape (dlg.Description.Text),
								dlg.ClassNumber.Value.ToString (), DB.Escape (dlg.Token.Text)));
				} else {
					DB.FastQuery (String.Format ("UPDATE classes SET token='{0}' WHERE id = {1};",
						DB.Escape (dlg.Token.Text), dlg.ClassID));
					DB.FastQuery (String.Format ("UPDATE classes SET number='{0}' WHERE id = {1};",
						dlg.ClassNumber.Value.ToString (), dlg.ClassID));
					DB.FastQuery (String.Format ("UPDATE classes SET description='{0}' WHERE id = {1};",
						DB.Escape (dlg.Description.Text), dlg.ClassID));
					DB.FastQuery (String.Format ("UPDATE classes SET year='{0}' WHERE id = {1};",
						dlg.SchoolYear.Value.ToString (), dlg.ClassID));
				}
				
				ClassesListReload (ShowOnlyCurrentSchoolYear.Active);
			}
			
			dlg.dialog.Destroy ();
			Tools.SetSensitive (window, true);
		}

 		
 		/// <summary>On remove class button clicked. Removes selected class.</summary>
		protected void on_RemoveClass_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (ClassesList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) ClassesList.Model;

				Tools.SetSensitive (window, false);
				MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, String.Format (Catalog.GetString ("Are you really sure to delete <i>{0}</i> class ?\n<b>All the students</b> in this class will be also removed !"),
											(string) store.GetValue (itr, 3)));
				
				if (ResponseType.Yes == (ResponseType) msg.Run ()) {
					ClassesButtonsSetSensitive (false);
										
					DB.FastQuery (String.Format ("DELETE FROM classes WHERE id = {0};", (int) store.GetValue (itr, 4)));
	

					IDataReader reader   = DB.Query (String.Format ("SELECT * FROM students WHERE class = '{0}';",
												(int) store.GetValue (itr, 4)));
					ArrayList students = new ArrayList ();

					if (reader != null) {
						while (reader.Read ())
							students.Add ((int) reader ["id"]);

						reader.Close ();
					}
					
					foreach (int id in students) {
						DB.FastQuery (String.Format ("DELETE FROM frequency WHERE student = {0};", id));
						DB.FastQuery (String.Format ("DELETE FROM marks WHERE student = {0};", id));
						DB.FastQuery (String.Format ("DELETE FROM remarks WHERE student = {0};", id));
						DB.FastQuery (String.Format ("DELETE FROM semestermarks WHERE student = {0};", id));
						DB.FastQuery (String.Format ("DELETE FROM students WHERE id = {0};", id));
					}

					store.Remove (ref itr);
				}

				Tools.SetSensitive (window, true);
				msg.Destroy ();

				return;
			}
		}


		/// <summary>On import classes button clicked. Shows import classes dialog.</summary>
		protected void on_ImportClassesBtn_clicked (object o, EventArgs args)
		{
			int SchoolYear = Tools.CurrentSchoolYear () - 1;
			ImportClassesDialog dlg = new ImportClassesDialog (this.window, SchoolYear);
			
			Tools.SetSensitive (window, false);

			if (dlg.dialog.Run () == (int) ResponseType.Ok)
				ClassesListReload (ShowOnlyCurrentSchoolYear.Active);

			dlg.dialog.Destroy ();
			Tools.SetSensitive (window, true);
		}


		/// <summary>On classes list row clicked.</summary>
		protected void on_ClassesList_row_activated (object o, RowActivatedArgs args)
		{
			on_OpenClass_clicked (null, null);
		}
		

		/// <summary>On open class button clicked.</summary>
		protected void on_OpenClass_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (ClassesList.Selection.GetSelected (out mod, out itr)) {
				ClassesButtonsSetSensitive (false);
				Tools.SetSensitive (window, false);
				
				ClassDialog dlg = new ClassDialog (this.window, (int) (mod as ListStore).GetValue (itr, 4));
				if (dlg.dialog.Run () == (int) ResponseType.Ok) {
					DB.FastQuery (String.Format ("UPDATE classes SET token='{0}' WHERE id = {1};",
							DB.Escape (dlg.Token.Text), dlg.ClassID));
					DB.FastQuery (String.Format ("UPDATE classes SET number='{0}' WHERE id = {1};",
						dlg.ClassNumber.Value.ToString (), dlg.ClassID));
					DB.FastQuery (String.Format ("UPDATE classes SET description='{0}' WHERE id = {1};",
						DB.Escape (dlg.Description.Text), dlg.ClassID));
					DB.FastQuery (String.Format ("UPDATE classes SET year='{0}' WHERE id = {1};",
						dlg.SchoolYear.Value.ToString (), dlg.ClassID));
				}
				
				dlg.dialog.Destroy ();
				ClassesListReload (ShowOnlyCurrentSchoolYear.Active);
				
				Tools.SetSensitive (window, true);
			}
		}


		/// <summary>On classes list cursor changed.</summary>
		protected void on_ClassesList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (ClassesList.Selection.GetSelected (out mod, out itr))
				ClassesButtonsSetSensitive (true);
			else
				ClassesButtonsSetSensitive (false);
		}
	}
}
