/*
 *  GUI/ModifyingSubjectDialog.cs
 *  Copyright (C) 2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Data;
using Mono.Unix;
using esofokles.Engine;
using esofokles.Widgets;
using Gtk;
using Glade;



namespace esofokles.GUI {

	/// <summary>ModifyingSubjectDialog dialog</summary>
	public class ModifyingSubjectDialog {
		public Gtk.Dialog dialog;
		
		
		[Glade.Widget] public  Gtk.Entry       Subject, Teacher;
		[Glade.Widget] public  Gtk.CheckButton Average;
		[Glade.Widget] private Gtk.Label       Title;
		private Button24 OkBtn;
		
		public int SubjectID, ClassID;

		
		/// <summary>Constructor. Creates ModifyingSubjectDialog.</summary>
		public ModifyingSubjectDialog (Gtk.Window parent, int subject, int ClassID)
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "ModifyingSubjectDialog", null);
			gui.Autoconnect (this);
		
			dialog = (Gtk.Dialog) gui.GetWidget ("ModifyingSubjectDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			dialog.DestroyWithParent = true;
			
			SubjectID    = subject;
			this.ClassID = ClassID;
			
			
			// OkBtn
			OkBtn = new Button24 (Catalog.GetString ("_OK"));
			OkBtn.Clicked += new EventHandler (on_OkBtn_clicked);
			dialog.ActionArea.Add (OkBtn);
			OkBtn.ShowAll ();
			
			
			// Subjects list
			if (SubjectID != -1) { // modifying
				Icons.AssignImage (Icons.MODIFY_SUBJECT_DIALOG, "Icon", gui);
				
				IDataReader reader = DB.Query (String.Format ("SELECT * FROM " +
									"subjects WHERE id = '{0}';", SubjectID));
				if (reader != null) {
					if (reader.Read ()) {
						Subject.Text   = (string) reader ["subject"];
						Teacher.Text   = (string) reader ["teacher"];
						Average.Active = ((int) reader ["average"] == 1);
					}
					
					reader.Close ();
				}
			} else { // adding
				Icons.AssignImage (Icons.ADD_SUBJECT_DIALOG, "Icon", gui);
				dialog.Title = Catalog.GetString ("Adding a new subject");
				Title.Markup = "<span size='x-large' weight='bold'>" +
						Catalog.GetString ("Adding a new subject") + "</span>";
			}
			
			
			// Icons and colors
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.OK_BUTTON, OkBtn.Img);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			
			Colors.Modify (Colors.HEAD, "Bg", gui);
			
			
			// Autocompletion
			Autocompletion.Assign (Subject, Field.SubjectsName);
			Autocompletion.Assign (Teacher, Field.SubjectsTeacher);
			
			
			SetSensitive ();
		}
		
		
		protected void on_OkBtn_clicked (object o, EventArgs args)
		{
			if (Subject.Text == "")
				return;
			
			Tools.SetSensitive (dialog, false);
			
			Autocompletion.AddText (Subject, Field.SubjectsName);
			Autocompletion.AddText (Teacher, Field.SubjectsTeacher);

			if (SubjectID == -1) {
				IDataReader reader = DB.Query (String.Format ("SELECT * FROM " +
								"subjects WHERE class = '{0}' AND subject = '{1}';",
								ClassID, DB.Escape (Subject.Text)));
				
				if (reader != null) {
					if (reader.Read ())
						if (Subject.Text == (string) reader ["subject"]) {
							MessageDialog md = new MessageDialog (dialog, 
								DialogFlags.DestroyWithParent, MessageType.Error,
								ButtonsType.Ok, Catalog.GetString ("The same " +
								"subject already exists !\nEnter other subject..."));
							
							md.Run ();
							md.Destroy ();
							Tools.SetSensitive (dialog, true);
							reader.Close ();
							
							return;
						}
					
					reader.Close ();
				}
			}
			
			dialog.Respond ((int) ResponseType.Ok);
		}
		
		
		protected void on_Subject_changed (object o, EventArgs args)
		{
			SetSensitive ();
		}


		protected void on_Subject_activate (object o, EventArgs args)
		{
			on_OkBtn_clicked (null, null);
		}


		protected void on_Teacher_changed (object o, EventArgs args)
		{
			SetSensitive ();
		}


		protected void on_Teacher_activate (object o, EventArgs args)
		{
			on_OkBtn_clicked (null, null);
		}
		
		
		/// <summary>Sets sensitive.</summary>
		private void SetSensitive ()
		{
			OkBtn.Sensitive = Subject.Text != "";
		}
	}
}
