/*
 *  GUI/AboutDialog.cs
 *  Copyright (C) 2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Data;
using System.Collections;
using Mono.Unix;
using Gdk;
using Gtk;
using Glade;
using esofokles.Engine;


namespace esofokles.GUI {

	/// <summary>Move students dialog</summary>
	public class MoveStudentDialog {
		public Gtk.Dialog dialog;

		[Glade.Widget] private Gtk.Button      MoveBtn, CancelBtn;
		[Glade.Widget] private Gtk.Label       StudentName;
		[Glade.Widget] public  Gtk.ComboBox    ClassesList, OtherClassesList;
		[Glade.Widget] public  Gtk.RadioButton MoveClass, MoveOtherClass;
		[Glade.Widget] private Gtk.Table       MoveClassTable, MoveOtherClassTable;
		[Glade.Widget] private Gtk.Frame       MoveClassFrame, MoveOtherClassFrame;
		[Glade.Widget] public  Gtk.CheckButton ImportMarks, ImportRemarks;
		[Glade.Widget] private Gtk.Notebook    Pages;

		
		public int ClassID;
		
		
		/// <summary>Constructor. Creates move student dialog</summary>
		public MoveStudentDialog (Gtk.Window parent, string Name, int ClassID)
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "MoveStudentDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("MoveStudentDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			dialog.DestroyWithParent = true;
			
			StudentName.Text = Name;
			this.ClassID = ClassID;
			
			// Icons and colors
			Icons.AssignWindow (dialog);
			
			Icons.AssignImage (Icons.MOVE_STUDENT_HEAD, "Icon", gui);
			Icons.AssignImage (Icons.MOVE_STUDENT_BUTTON, "MoveBtnImg", gui);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			
			Colors.Modify (Colors.HEAD, "Bg", gui);


			// class information
			IDataReader reader1 = DB.Query (String.Format ("SELECT * FROM classes " +
												"WHERE id = '{0}';", ClassID));
			
			string ClassToken = "";
			int    Year = 0, ClassNumber = 0;
			
			if (reader1 != null) {
				reader1.Read ();
				ClassNumber = (int) reader1 ["number"];
				ClassToken  = (string) reader1 ["token"];
				Year        = (int) reader1 ["year"];

				reader1.Close ();
			}

			
			// classes lists
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM classes " +
													"WHERE year = '{0}';", Year));
			
			if (reader != null) {
				while (reader.Read ())
					if ((int) reader ["number"] == ClassNumber) {
						if ((string) reader ["token"] != ClassToken)
							ClassesList.AppendText (String.Format ("{0} {1}",
								(int) reader ["number"], (string) reader ["token"]));
					} else
						OtherClassesList.AppendText (String.Format ("{0} {1}",
							(int) reader ["number"], (string) reader ["token"]));

				reader.Close ();
			}
			
			
			// classes lists - sensitive
			MoveClassFrame.Sensitive      = ClassesList.Model.IterNChildren () > 0;
			MoveOtherClass.Active         = ! MoveClassFrame.Sensitive;
			MoveOtherClassFrame.Sensitive = OtherClassesList.Model.IterNChildren () > 0;
			
			if (ClassesList.Model.IterNChildren () > 0)
				ClassesList.Active = 0;
			
			if (OtherClassesList.Model.IterNChildren () > 0)
				OtherClassesList.Active = 0;
			
			if ((MoveClassFrame.Sensitive == false) && (MoveOtherClassFrame.Sensitive == false)) {
				Pages.Page = 1;
				MoveBtn.Hide ();
				CancelBtn.Label = Catalog.GetString ("_Close");
			} else
				Pages.Page = 0;
		}


		protected void on_MoveClass_toggled (object o, EventArgs args)
		{
			if (MoveClass.Active) {
				MoveClassTable.Sensitive      = true;
				MoveOtherClassTable.Sensitive = false;
			} else {
				MoveClassTable.Sensitive      = false;
				MoveOtherClassTable.Sensitive = true;
			}
		}

		
		protected void on_ClassesList_changed (object o, EventArgs args)
		{
			Gtk.TreeIter tmp;
			MoveBtn.Sensitive = ClassesList.Model.GetIterFirst (out tmp);
		}


		protected void on_OtherClassesList_changed (object o, EventArgs args)
		{ 
			Gtk.TreeIter tmp;
			MoveBtn.Sensitive = OtherClassesList.Model.GetIterFirst (out tmp);
		}
	}
}