/*
 *  GUI/SubjectsListDialog.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Data;
using System.Data.SqlTypes;
using Gtk;
using esofokles.Engine;
using Mono.Unix;


namespace esofokles.GUI {
	
	/*
	 * SubjectsListDialog class
	 */
	
	public class SubjectsListDialog {
		public  Gtk.Dialog dialog;
		private Glade.XML  gui;

		[Glade.Widget] public  Gtk.TreeView SubjectsList;
		[Glade.Widget] private Gtk.Button   AddBtn, RemoveBtn, ModifyBtn;

		int ClassID;


		public SubjectsListDialog (Gtk.Window parent, int ClassID)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "SubjectsListDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("SubjectsListDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			dialog.DestroyWithParent = true;
			
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.ADD_SUBJECT_BUTTON, "AddBtnImg", gui);
			Icons.AssignImage (Icons.REMOVE_SUBJECT_BUTTON, "RemoveBtnImg", gui);
			Icons.AssignImage (Icons.SAVE_SUBJECT_BUTTON, "ModifyBtnImg", gui);
			Icons.AssignImage (Icons.OK_BUTTON, "OkBtnImg", gui);
			Icons.AssignImage (Icons.SUBJECTS_LIST_DIALOG, "Icon", gui);
			
			Colors.Modify (Colors.HEAD, "Bg", gui);
			
			Tools.SetSensitive (dialog, false);
			
			this.ClassID  = ClassID;
			SubjectsListCreate ();
			ReloadSubjectsList ();
			
			Tools.SetSensitive (dialog, true);
		}


		protected void on_SubjectsList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			RemoveBtn.Sensitive = SubjectsList.Selection.GetSelected (out mod, out itr);
			ModifyBtn.Sensitive = SubjectsList.Selection.GetSelected (out mod, out itr);
		}
		
		
		protected void on_SubjectsList_row_activated (object o,  RowActivatedArgs args)
		{
			on_ModifyBtn_clicked (null, null);
		}


		protected void on_AddBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			
			ModifyingSubjectDialog dlg = new ModifyingSubjectDialog (dialog, -1, ClassID);
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				DB.FastQuery (String.Format ("INSERT INTO subjects (class, subject, teacher, average) VALUES (" +
						"'{0}', '{1}', '{2}', '{3}');", ClassID, DB.Escape (dlg.Subject.Text),
						DB.Escape (dlg.Teacher.Text), Convert.ToInt32 (dlg.Average.Active)));
				ReloadSubjectsList ();
			}
			
			dlg.dialog.Destroy ();
			Tools.SetSensitive (dialog, true);
			dialog.Focus = AddBtn;
		}
		
		
		protected void on_ModifyBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (SubjectsList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				
				ModifyingSubjectDialog dlg = new ModifyingSubjectDialog (dialog,
					(int) (SubjectsList.Model as ListStore).GetValue (itr, 2), ClassID);
				if (dlg.dialog.Run () == (int) ResponseType.Ok) {
					string query = "";
					query += String.Format ("UPDATE subjects SET subject = '{0}' WHERE id = '{1}';",
								DB.Escape (dlg.Subject.Text), dlg.SubjectID);
					query += String.Format ("UPDATE subjects SET teacher = '{0}' WHERE id = '{1}';",
								DB.Escape (dlg.Teacher.Text), dlg.SubjectID);
					query += String.Format ("UPDATE subjects SET average = '{0}' WHERE id = '{1}';",
								Convert.ToInt32 (dlg.Average.Active), dlg.SubjectID);
					DB.FastQuery (query);
					
					ReloadSubjectsList ();
				}
				
				dlg.dialog.Destroy ();
				
				Tools.SetSensitive (dialog, true);
				dialog.Focus = SubjectsList;
			}
		}


		protected void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (SubjectsList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				
				MessageDialog md = new MessageDialog (dialog, DialogFlags.DestroyWithParent, MessageType.Warning, ButtonsType.YesNo,
						Catalog.GetString ("Are you really sure to delete " +
						"selected subject ?!\nAll the marks of students from " +
						"this class will be deleted, too !"));
				if (md.Run () == (int) ResponseType.Yes) {
					DB.FastQuery (String.Format ("DELETE FROM subjects WHERE id = '{0}';",
							mod.GetValue (itr, 2).ToString ()));
					
					DB.FastQuery (String.Format ("DELETE FROM marks WHERE " +
						"subject = '{0}';", mod.GetValue (itr, 2).ToString ()));
					DB.FastQuery (String.Format ("DELETE FROM semestermarks " +
						"WHERE subject = '{0}';", mod.GetValue (itr, 2).ToString ()));
					
					(mod as ListStore).Remove (ref itr);
				}
				
				md.Destroy();
				ReloadSubjectsList ();
				Tools.SetSensitive (dialog, true);
				
				dialog.Focus = SubjectsList;
			}
		}


		/// <summary>Creates subjects list.</summary>
		private void SubjectsListCreate ()
		{
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (string), typeof (int));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			SubjectsList.Model = store;

			SubjectsList.AppendColumn (Catalog.GetString ("Subject"), new CellRendererText  (), "text", 0);
			SubjectsList.AppendColumn (Catalog.GetString ("Teacher"), new CellRendererText (), "text", 1);

			SubjectsList.Columns [0].Clickable = true;
			SubjectsList.Columns [0].SortIndicator = true;
			SubjectsList.Columns [0].SortColumnId = 0;

			SubjectsList.Columns [1].Clickable = true;
			SubjectsList.Columns [1].SortIndicator = true;
			SubjectsList.Columns [1].SortColumnId = 1;
		}


		/// <summary>Reloads subjects list.</summary>
		private void ReloadSubjectsList ()
		{
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM subjects WHERE class = '{0}';", ClassID));
			
			if (reader != null) {
				Gtk.ListStore store = (Gtk.ListStore) SubjectsList.Model;
				store.Clear ();
				
				while (reader.Read ())
					store.AppendValues (new object [] {(string) reader ["subject"], (string) reader ["teacher"], (int) reader ["id"]});

				reader.Close ();
			}
			
			RemoveBtn.Sensitive = false;
			ModifyBtn.Sensitive = false;
		}
	}
}
