/*
 *  Widgets/DateEdit.cs
 *  Copyright (C) 2004-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using GLib;


namespace esofokles.Widgets {
	public class DateEdit : Gtk.HBox {
		private Gtk.Entry        DateEntry;
		private Gtk.ToggleButton SelectBtn;
		private Gtk.Window       CalendarWindow;
		private Gtk.Calendar     Calendar;
		
		
		public DateEdit ()
		{
			Gtk.HBox MainBox = new Gtk.HBox ();
			this.Add (MainBox);
			
			// This
			MainBox.Spacing = 1;
			
			
			// CalendarWindow
			CalendarWindow = new Gtk.Window (" ");
			CalendarWindow.TypeHint = Gdk.WindowTypeHint.Dialog;
			CalendarWindow.Decorated = false;
			CalendarWindow.WindowPosition = Gtk.WindowPosition.Mouse;
			CalendarWindow.Modal = true;
			CalendarWindow.FocusOutEvent += new Gtk.FocusOutEventHandler (on_CalendarWindow_focus_out);
			
			Gtk.Viewport VP = new Gtk.Viewport ();
			
			Gtk.VBox CWBox = new Gtk.VBox ();
			CWBox.Spacing  = 3;
			
			this.Calendar = new Gtk.Calendar ();
			this.Calendar.Show ();
			this.Calendar.Date = System.DateTime.Today;
			this.Calendar.DaySelectedDoubleClick += new EventHandler (on_Calendar_day_selected_double_clicked);
			CWBox.Add (this.Calendar);
			
			Gtk.HButtonBox BBox = new Gtk.HButtonBox ();
			BBox.BorderWidth = 6;
			BBox.Spacing = 6;
			CWBox.Add (BBox);
			
			Gtk.Button OkBtn = new Gtk.Button ("OK");
			OkBtn.Clicked += new EventHandler (on_OkBtn_clicked);
			BBox.Add (OkBtn);
			
			Gtk.Button CancelBtn = new Gtk.Button ("Anuluj");
			CancelBtn.Clicked += new EventHandler (on_CancelBtn_clicked);
			BBox.Add (CancelBtn);
			
			VP.Add (CWBox);
			VP.ShowAll ();
			CalendarWindow.Add (VP);
			
			
			// DateEntry
			DateEntry = new Gtk.Entry ();
			DateEntry.WidthRequest = 70;
			DateEntry.IsEditable = false;
			DateEntry.Text = FormatDate (this.Calendar.Date);
			DateEntry.Show ();
			MainBox.PackStart (DateEntry, true, true, 1);
			
			
			// SelectBtn
			SelectBtn = new Gtk.ToggleButton ();
			SelectBtn.Add (new Gtk.Image (Gtk.Stock.GoDown, Gtk.IconSize.Menu));
			SelectBtn.Clicked += new EventHandler (on_SelectBtn_clicked);
			MainBox.PackEnd (SelectBtn, false, true, 0);
			
			
			this.ShowAll ();
		}
		
		
		/* EVENTS *************************************************************/
		private void on_SelectBtn_clicked (object obj, EventArgs args)
		{
			Gtk.ToggleButton btn = (Gtk.ToggleButton) obj;
			
			if (btn.Active == true)
				CalendarWindow.Show ();
			else
				CalendarWindow.Hide ();
		}
		
		
		private void on_OkBtn_clicked (object o, EventArgs args)
		{
			DateEntry.Text = FormatDate (this.Calendar.Date);
			CalendarWindow.Hide ();
			SelectBtn.Active = false;
		}
		
		
		private void on_CancelBtn_clicked (object o, EventArgs args)
		{
			CalendarWindow.Hide ();
			SelectBtn.Active = false;
		}
		
		
		private void on_CalendarWindow_focus_out (object obj, Gtk.FocusOutEventArgs args)
		{
			CalendarWindow.Hide ();
			SelectBtn.Active = false;
		}
		
		
		private void on_Calendar_day_selected_double_clicked (object o, EventArgs args)
		{
			DateEntry.Text = FormatDate (this.Calendar.Date);
			CalendarWindow.Hide ();
			SelectBtn.Active = false;
		}
		
		
		/* PRIVATE FUNCTIONS **************************************************/
		public string FormatDate (System.DateTime date)
		{
			return date.ToString ("dd-MM-yyyy");
		}
		
		
		/* PUBLIC FUNCTIONS ***************************************************/
		public void SetDate (System.DateTime date)
		{
			this.Calendar.Date = date;
			DateEntry.Text = FormatDate (date);
		}


		public void SetDate (string date)
		{
			try {
				string [] dt = date.Split ('-');
				this.Calendar.Date =  new DateTime (Convert.ToInt32 (dt [2]), Convert.ToInt32 (dt [1]), Convert.ToInt32 (dt [0]));
				DateEntry.Text = FormatDate (this.Calendar.Date);
			} catch {
				SetDate (DateTime.Today);
			}
		}
		
		
		public string GetDateString ()
		{
			return FormatDate (this.Calendar.Date);
		}
		
		
		public DateTime GetDate ()
		{
			return this.Calendar.Date;
		}
	}
}
